#!/usr/bin/env python3
import subprocess
import threading
import tkinter as tk
from time import sleep

# customize here
adb    = "adb"
serial = ""
scaler = 3

# adb remote class
class AdbRemote():
    def __init__(self, adb, serial):
        self.keys = {
            "Menu":1, "Home":3, "Back":4,
            "Up":19, "Down":20, "Left":21, "Right":22, "Center":23,
            "Vol+":24, "Vol-":25, "Power":26, "Camera":27, "unlock":82
        }
        if not serial:
            self.cmd = [adb]
        else:
            self.cmd = [adb, "-s", serial]

    def CheckDevice(self):
        try:
            out = subprocess.check_output(self.cmd + ["get-state"], stderr=subprocess.STDOUT)
            if "device" not in out.decode():
                return False
        except:
            return False
        return True

    def GetScreenshot(self):
        png = subprocess.check_output(self.cmd + ["exec-out", "screencap", "-p"])
        return png

    def SendTap(self, x, y):
        subprocess.check_call(self.cmd + ["shell", "input", "tap", "%d" % x, "%d" % y])

    def SendSwipe(self, down_x, down_y, up_x, up_y):
        subprocess.check_call(self.cmd + ["shell", "input", "swipe",
            "%d" % down_x, "%d" % down_y, "%d" % up_x, "%d" % up_y])

    def SendSpecialKey(self, key):
        if self.keys[key]:
            subprocess.check_call(self.cmd + ["shell", "input", "keyevent", "%d" % self.keys[key]])
        else:
            print("Unknown key %s" % key)

# main application
class App(tk.Frame):
    def __init__(self, master=None):
        tk.Frame.__init__(self, master)
        top = self.winfo_toplevel()
        top.columnconfigure(0, weight=1, minsize=300)
        top.rowconfigure(0, weight=1, minsize=300)
        self.pack()

        # top-level widgets
        self.keysFrame = tk.Frame(self)
        self.screenshot = tk.Label(self)
        self.hoverLabel = tk.Label(self, font=("Courier", 12), anchor=tk.NW, justify=tk.LEFT)
        self.keysFrame.pack()
        self.screenshot.pack(padx=4, pady=4)
        self.hoverLabel.pack(fill="x")

        # key widgets
        for key in ["Back", "Home", "Menu", "Vol+", "Vol-", "Camera", "Power", "unlock"]:
            tk.Button(self.keysFrame, text=key, command=lambda key=key: self.keyButton(key)).pack(side=tk.LEFT)

        # event handlers
        self.screenshot.bind("<Motion>", self.mouseMove)
        self.screenshot.bind("<ButtonPress>", self.mouseDown)
        self.screenshot.bind("<ButtonRelease>", self.mouseUp)
        self.bind("<<ImageUpdate>>", self.imageUpdate)

        # show main window
        self.screenshot["background"] = "gray"
        self.screenshot["text"] = "Connecting..."

        # start background thread
        self.bgThread = threading.Thread(target=self.AdbRemoteThread)
        self.bgThread.daemon = True
        self.bgThread.start()

    def mouseMove(self, event):
        if not self.adbremote.ready or not self.img:
            return

        #print("move at %d %d" % (event.x, event.y))
        try:
            pixel = self.img.get(event.x, event.y)
            rgb = pixel[0] * 65536 + pixel[1] * 256 + pixel[2]
            self.hoverLabel["text"] = "X=%04d Y=%04d: RGB=0x%08x (%3d,%3d,%3d)" % (
                scaler * event.x, scaler * event.y, rgb, pixel[0], pixel[1], pixel[2])
        except:
            self.hoverLabel["text"] = ""

    def mouseDown(self, event):
        #print("down at %d %d" % (event.x, event.y))
        self.downEvent = event

    def mouseUp(self, event):
        if not self.adbremote.ready:
            print("ignoring mouse: adb not ready")
            return

        #print("up at %d %d" % (event.x, event.y))
        if self.downEvent.x == event.x and self.downEvent.y == event.y:
            print("tap at %d %d" % (event.x * scaler, event.y * scaler))
            self.adbremote.SendTap(event.x * scaler, event.y * scaler)
        else:
            print("swipe from %d %d to %d %d" % (
                self.downEvent.x * scaler, self.downEvent.y * scaler,
                event.x * scaler, event.y * scaler))
            self.adbremote.SendSwipe(
                self.downEvent.x * scaler, self.downEvent.y * scaler,
                event.x * scaler, event.y * scaler)

    def keyButton(self, key):
        if not self.adbremote.ready:
            print("ignoring keypress: adb not ready")
            return

        print("send special key %s" % key)
        self.adbremote.SendSpecialKey(key)

    def imageUpdate(self, event):
        self.img = tk.PhotoImage(data=self.adbremote.png).subsample(scaler, scaler)
        self.screenshot["image"] = self.img

    def AdbRemoteThread(self):
        # create adb remote and wait until connected
        self.adbremote = AdbRemote(adb, serial)
        self.adbremote.ready = False
        while not self.adbremote.CheckDevice():
            sleep(1)

        # ready, keep updating screen
        self.adbremote.ready = True
        while True:
            self.adbremote.png = self.adbremote.GetScreenshot()
            self.event_generate("<<ImageUpdate>>", when="tail")

app = App()
app.master.title("ADB Remote")
app.mainloop()
